# -*- perl -*-

# Copyright (c) 2010 by Jeff Weisberg
# Author: Jeff Weisberg <jaw @ tcp4me.com>
# Created: 2010-Nov-22 13:24 (EST)
# Function: 
#
# $Id$

package AC::DC::IO::UDP;

use AC::DC::Debug 'udp';
use AC::DC::IO::UDP::Server;

use strict;

our @ISA = 'AC::DC::IO';



1;
